import os

from PyQt5.QtCore import QTranslator, QSettings, Qt, QCoreApplication
from PyQt5.QtWidgets import QMenu, QWidget, QSizePolicy, QApplication
from PyQt5 import QtWidgets
from PyQt5.QtCore import QCoreApplication
from qgis.PyQt import QtGui
from qgis.PyQt.QtGui import QAction
from qgis.utils import iface

from .resources import *  # 初始化资源，没有这句，图标不会显示

from .cyanlove_import_cell_neight_tosqlite import cyanlove_import_cell_neight_tosqlite
from .cyanlove_customdraw import cyanlove_customdraw
from .cyanlove_import_create_point import cyanlove_createpoint
from .cyanlove_import_create_geometry import cyanlove_import_geometry
from .cyanlove_import_create_mutiline import cyanlove_import_create_mutiline
from .cyanlove_export_wkt import cyanlove_exportwkt
from .cyanlove_sqliteset import cyanlove_sqliteset
from .cyanlove_dockwidget import cyanloveDockWidget
from .cyanlove_export_sheetattribute import cyanlove_export_sheetattribute
from .cyanlove_import_csv_excel_sqlite import cyanlove_import_csv_excel_sqlite
from .cyanlove_zoom_to_longitude_latitude import cyanlove_zoom_to_longitude_latitude
from .cyanlove_tencent_keyset import cyanlove_tencent_keyset
from .cyanlove_cell_nodeb_search import cyanlove_cell_nodeb_search
from .cyanlove_export_tab_kml import cyanlove_export_tab_kml
from .cyanlove_buffer import cyanlove_buffer
from .cyanlove_import_layerattribute_tosqlite import cyanlove_import_layerattribute_tosqlite
from .cyanlove_calc_area import cyanlove_calc_area
from .cyanlove_calc_intersects_area import cyanlove_calc_intersects_area
from .cyanlove_calc_intersects_geometry import cyanlove_calc_intersects_geometry
from .cyanlove_calc_contain_point_in_geometry import cyanlove_calc_contain_point_in_geometry
from .cyanlove_calc_dissolve import cyanlove_calc_dissolve


class cyanlove:

    def __init__(self, iface):

        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'cyanlove_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        self.menu = None  # 用于存储主菜单
        self.actions = []  # 用于存储添加的动作

        self.dockwidget_show_shange_neigth = None
        self.dockwidget_export_wkt_tocsv = None
        self.dockwidget_sqlite = None
        self.dockwidget_import_cell_neight_tosqlite = None
        self.dockwidget_import_csv_excel_tosqlite = None
        self.dockwidget_draw_polygon = None
        self.dockwidget_draw_mutiline = None
        self.dockwidget_custom_draw_polygon = None
        self.dockwidget_draw_point = None
        self.dockwidget_export_attribute = None
        self.dockwidget_zoom_to_longitude_latitude = None
        self.dockwidget_tencent_keyset = None
        self.dockwidget_cell_nodeb_search = None
        self.dockwidget_export_tab_kml = None
        self.dockwidget_buffer = None
        self.dockwidget_import_layerattribute_tosqlite = None
        self.dockwidget_calc_area = None
        self.dockwidget_calc_intersects_area = None
        self.dockwidget_calc_intersects_geometry = None
        self.dockwidget_calc_contain_point_in_geometry = None
        self.dockwidget_calc_dissolve = None

    def initGui(self):
        # 创建主菜单 "申少军工具箱"
        self.menu = QtWidgets.QMenu(QCoreApplication.translate("cyanlove", "申少军工具箱"))
        self.iface.mainWindow().menuBar().addMenu(self.menu)
        icon_path1 = ':/plugins/cyanlove/icon/accessibility-svgrepo-com.svg'
        icon_path2 = ':/plugins/cyanlove/icon/activity-svgrepo-com.svg'

        # 定义子菜单项
        db_submenu = [
            ("Sqlite数据库配置", self.sub_sqlite, icon_path1),
            ("腾讯Key值配置", self.sub_tencent_keyset, icon_path2)
        ]
        export_submenu = [
            ("边界经纬度集导出", self.sub_export_wkt_tocsv, icon_path1),
            ("属性表信息导出", self.sub_export_attribute, icon_path2),
            ("导出为TAB格式", self.sub_export_TAB_kml_file, icon_path2),
            ("导出为谷歌KML格式", self.sub_export_TAB_kml_file, icon_path2)
        ]
        updatedata_submenu = [
            ("更新", self.sub_analysis2, icon_path1),
            ("过滤", self.sub_analysis2, icon_path2)
        ]
        import_submenu = [
            ("小区表/邻区表CSV/Excel导入(Sqlite)", self.sub_import_cell_neigth_file_tosqlite, icon_path1),
            ("其他文件CSV/Excel导入(Sqlite)", self.sub_import_csv_excel_tosqlite, icon_path2),
            ("表选择列创建索引(Sqlite)", self.sub_create_index_colname_sqlite, icon_path2),
            ("选择图层数据导入(Sqlite)", self.sub_import_layerattribute_tosqlite, icon_path1)
        ]
        create_polygon_submenu = [
            ("创建点图层", self.sub_draw_point, icon_path1),
            ("创建线图层", self.sub_draw_mutiline, icon_path2),
            ("创建面图层", self.sub_draw_polygon, icon_path1),
            ("自定义点、线、面图层=>(逗号,分号)或WKT格式", self.sub_custom_draw_polygon, icon_path1),
            ("创建扇区图层", self.sub_analysis2, icon_path1),
        ]
        zoom_location_submenu = [
            ("坐标定位/地理搜索", self.sub_zoom_to_longitude_latitude, icon_path1),
            ("小区/基站搜索定位", self.sub_cell_nodeb_search, icon_path1)
        ]
        wangluoyouhua_submenu = [
            ("栅格/邻区显示(PostGreSQL)", self.sub_show_shange_neigth, icon_path1),
            ("栅格/邻区显示(Sqlite)", self.sub_show_shange_neigth, icon_path1),
        ]
        diliwenxi_submenu = [
            ("缓冲区分析", self.sub_buffer_meters, icon_path1),
            ("面积计算", self.sub_calc_area, icon_path1),
            ("相交面积计算", self.sub_calc_intersects_area, icon_path1),
            ("图层合并", self.sub_analysis2, icon_path1),
            ("关系计算_包含", self.sub_calc_contain_point_in_geometry, icon_path1),
            ("关系计算_相交", self.sub_calc_intersects_geometry, icon_path1),
            ("关系计算_相交合并", self.sub_calc_dissolve, icon_path1),
            ("关系计算_不相交分离", self.sub_analysis2, icon_path1)
        ]
        transform_submenu = [
            ("WGS84 -> 腾讯/高德", self.sub_analysis2, icon_path1),
            ("WGS84 -> 百度", self.sub_analysis2, icon_path1),
        ]
        cluster_submenu = [
            ("DBSCAN聚类", self.sub_analysis2, icon_path1),
            ("最近邻分析", self.sub_analysis2, icon_path1),
            ("K均值聚类", self.sub_analysis2, icon_path1),
        ]
        # 添加菜单项和子菜单

        self.add_menu_actions("数据配置", db_submenu)
        self.add_menu_actions("数据导出", export_submenu)
        self.add_menu_actions("数据入库", import_submenu)
        self.add_menu_actions("数据更新", updatedata_submenu)

        # 添加分隔符
        separator = QAction("1.", self.menu)
        separator.setSeparator(True)  # 设置为分隔符
        self.menu.addAction(separator)
        self.add_menu_actions("创建图层", create_polygon_submenu)
        self.add_menu_actions("定位/搜索", zoom_location_submenu)
        self.add_menu_actions("图形处理分析", diliwenxi_submenu)
        self.add_menu_actions("网络优化", wangluoyouhua_submenu)
        # 添加分隔符
        separator = QAction("2.", self.menu)
        separator.setSeparator(True)  # 设置为分隔符
        self.menu.addAction(separator)
        self.add_menu_actions("坐标系转换", transform_submenu)
        self.add_menu_actions("聚类分析", cluster_submenu)

    def add_menu_actions(self, menu_name, submenu_items):
        """添加菜单项及其子菜单"""
        # 创建菜单项
        action = QtWidgets.QAction(QCoreApplication.translate("cyanlove", menu_name),
                                   self.iface.mainWindow())
        self.menu.addAction(action)
        self.actions.append(action)  # 保存动作

        # 创建子菜单
        submenu = self.create_submenu(menu_name, submenu_items)
        action.setMenu(submenu)

    def create_submenu(self, submenu_name, actions):
        """创建子菜单的通用函数"""
        submenu = QtWidgets.QMenu(QCoreApplication.translate("cyanlove", submenu_name), self.iface.mainWindow())
        for action_name, method, icon_path in actions:
            self.add_submenu_action(submenu, action_name, method, icon_path)
        return submenu

    def add_submenu_action(self, submenu, action_name, method, icon_path):
        """添加一个子菜单项的通用函数，带图标"""
        action = QtWidgets.QAction(QCoreApplication.translate("cyanlove", action_name), self.iface.mainWindow())
        action.triggered.connect(method)  # 连接到指定的方法
        action.setIcon(QtGui.QIcon(icon_path))  # 添加图标
        submenu.addAction(action)  # 将动作添加到子菜单

    # 栅格/邻区显示
    def sub_show_shange_neigth(self):
        # 这里添加子菜单分析1的逻辑
        if self.dockwidget_show_shange_neigth is None:
            self.dockwidget_show_shange_neigth = cyanloveDockWidget()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_show_shange_neigth)
            # 设置 Dock Widget 的几何位置
            self.dockwidget_show_shange_neigth.setGeometry(1300, 530, 200, 150)  # 设置初始位置和大小
        # 每次点击时，显示 dockwidget
        self.dockwidget_show_shange_neigth.show()
        # 设置为浮动
        self.dockwidget_show_shange_neigth.setFloating(True)

    # 小区/基站信息搜索
    def sub_cell_nodeb_search(self):
        # 这里添加子菜单分析1的逻辑
        if self.dockwidget_cell_nodeb_search is None:
            self.dockwidget_cell_nodeb_search = cyanlove_cell_nodeb_search()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_cell_nodeb_search)
            self.dockwidget_cell_nodeb_search.setGeometry(1230, 460, 270, 150)  # 设置初始位置和大小
        # 每次点击时，显示 dockwidget
        self.dockwidget_cell_nodeb_search.show()
        # 设置为浮动
        self.dockwidget_cell_nodeb_search.setFloating(True)

    # 导出WKT格式边界集
    def sub_export_wkt_tocsv(self):
        if self.dockwidget_export_wkt_tocsv is None:
            self.dockwidget_export_wkt_tocsv = cyanlove_exportwkt()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_export_wkt_tocsv)
            self.dockwidget_export_wkt_tocsv.setGeometry(500, 400, 600, 160)

        # 每次点击时，显示 dockwidget
        self.dockwidget_export_wkt_tocsv.show()
        # 设置为浮动
        self.dockwidget_export_wkt_tocsv.setFloating(True)

    # sqlite数据库配置
    def sub_sqlite(self):
        if self.dockwidget_sqlite is None:
            self.dockwidget_sqlite = cyanlove_sqliteset()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_sqlite)
        # 每次点击时，显示 dockwidget
        self.dockwidget_sqlite.show()

    # 导入小区和邻区表到sqlite数据库
    def sub_import_cell_neigth_file_tosqlite(self):
        if self.dockwidget_import_cell_neight_tosqlite is None:
            self.dockwidget_import_cell_neight_tosqlite = cyanlove_import_cell_neight_tosqlite()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_import_cell_neight_tosqlite)
            self.dockwidget_import_cell_neight_tosqlite.setGeometry(400, 300, 700, 180)
        # 每次点击时，显示 dockwidget
        self.dockwidget_import_cell_neight_tosqlite.show()
        # 设置为浮动
        self.dockwidget_import_cell_neight_tosqlite.setFloating(True)

    # 导入csv/excel到sqlite数据库
    def sub_import_csv_excel_tosqlite(self):
        if self.dockwidget_import_csv_excel_tosqlite is None:
            self.dockwidget_import_csv_excel_tosqlite = cyanlove_import_csv_excel_sqlite()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_import_csv_excel_tosqlite)
            self.dockwidget_import_csv_excel_tosqlite.setGeometry(400, 300, 700, 180)
        # 每次点击时，显示 dockwidget
        self.dockwidget_import_csv_excel_tosqlite.show()
        # 设置为浮动
        self.dockwidget_import_csv_excel_tosqlite.setFloating(True)

    # 选择csv/excel文件绘制多边形面
    def sub_draw_polygon(self):
        if self.dockwidget_draw_polygon is None:
            self.dockwidget_draw_polygon = cyanlove_import_geometry()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_draw_polygon)
            self.dockwidget_draw_polygon.setGeometry(400, 300, 600, 240)
            # 每次点击时，显示 dockwidget
        self.dockwidget_draw_polygon.show()
        # 设置为浮动
        self.dockwidget_draw_polygon.setFloating(True)

    # 选择csv/excel文件绘制多条线
    def sub_draw_mutiline(self):
        if self.dockwidget_draw_mutiline is None:
            self.dockwidget_draw_mutiline = cyanlove_import_create_mutiline()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_draw_mutiline)
            self.dockwidget_draw_mutiline.setGeometry(400, 300, 600, 240)
            # 每次点击时，显示 dockwidget
        self.dockwidget_draw_mutiline.show()
        # 设置为浮动
        self.dockwidget_draw_mutiline.setFloating(True)

    # 输入经纬度集绘制点，线，面
    def sub_custom_draw_polygon(self):
        if self.dockwidget_custom_draw_polygon is None:
            self.dockwidget_custom_draw_polygon = cyanlove_customdraw()
            self.iface.addDockWidget(Qt.RightDockWidgetArea, self.dockwidget_custom_draw_polygon)
            # 每次点击时，显示 dockwidget
        self.dockwidget_custom_draw_polygon.show()

    # 定位
    def sub_zoom_to_longitude_latitude(self):
        if self.dockwidget_zoom_to_longitude_latitude is None:
            self.dockwidget_zoom_to_longitude_latitude = cyanlove_zoom_to_longitude_latitude()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_zoom_to_longitude_latitude)
            # 每次点击时，显示 dockwidget
        self.dockwidget_zoom_to_longitude_latitude.show()

    # 选择点文件，创建点图层
    def sub_draw_point(self):
        if self.dockwidget_draw_point is None:
            self.dockwidget_draw_point = cyanlove_createpoint()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_draw_point)
            self.dockwidget_draw_point.setGeometry(400, 300, 600, 140)
            # 每次点击时，显示 dockwidget
        self.dockwidget_draw_point.show()
        # 设置为浮动
        self.dockwidget_draw_point.setFloating(True)

    # 导出图层属性表
    def sub_export_attribute(self):
        if self.dockwidget_export_attribute is None:
            self.dockwidget_export_attribute = cyanlove_export_sheetattribute()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_export_attribute)
        # 每次点击时，显示 dockwidget
        self.dockwidget_export_attribute.show()

    # 腾讯key值设置
    def sub_tencent_keyset(self):
        if self.dockwidget_tencent_keyset is None:
            self.dockwidget_tencent_keyset = cyanlove_tencent_keyset()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_tencent_keyset)
        # 每次点击时，显示 dockwidget
        self.dockwidget_tencent_keyset.show()

    # 导出mapinfo TAB或者KML文件
    def sub_export_TAB_kml_file(self):
        if self.dockwidget_export_tab_kml is None:
            self.dockwidget_export_tab_kml = cyanlove_export_tab_kml()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_export_tab_kml)
            # 每次点击时，显示 dockwidget
        self.dockwidget_export_tab_kml.show()

    # 实现米的缓冲
    def sub_buffer_meters(self):
        if self.dockwidget_buffer is None:
            self.dockwidget_buffer = cyanlove_buffer()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_buffer)
            # 每次点击时，显示 dockwidget
        self.dockwidget_buffer.show()

    # 实现图层数据入库
    def sub_import_layerattribute_tosqlite(self):
        if self.dockwidget_import_layerattribute_tosqlite is None:
            self.dockwidget_import_layerattribute_tosqlite = cyanlove_import_layerattribute_tosqlite()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_import_layerattribute_tosqlite)
            self.dockwidget_import_layerattribute_tosqlite.setGeometry(500, 400, 600, 160)
            # 每次点击时，显示 dockwidget
        self.dockwidget_import_layerattribute_tosqlite.show()
        # 设置为浮动
        self.dockwidget_import_layerattribute_tosqlite.setFloating(True)

    # 面积计算
    def sub_calc_area(self):
        if self.dockwidget_calc_area is None:
            self.dockwidget_calc_area = cyanlove_calc_area()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_calc_area)
            self.dockwidget_calc_area.setGeometry(500, 300, 600, 160)
            # 每次点击时，显示 dockwidget
        self.dockwidget_calc_area.show()
        # 设置为浮动
        self.dockwidget_calc_area.setFloating(True)

    # 相交面积计算
    def sub_calc_intersects_area(self):
        if self.dockwidget_calc_intersects_area is None:
            self.dockwidget_calc_intersects_area = cyanlove_calc_intersects_area()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_calc_intersects_area)
            self.dockwidget_calc_intersects_area.setGeometry(500, 300, 600, 160)
            # 每次点击时，显示 dockwidget
        self.dockwidget_calc_intersects_area.show()
        # 设置为浮动
        self.dockwidget_calc_intersects_area.setFloating(True)

    # 相交计算
    def sub_calc_intersects_geometry(self):
        if self.dockwidget_calc_intersects_geometry is None:
            self.dockwidget_calc_intersects_geometry = cyanlove_calc_intersects_geometry()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_calc_intersects_geometry)
            self.dockwidget_calc_intersects_geometry.setGeometry(500, 300, 600, 160)
            # 每次点击时，显示 dockwidget
        self.dockwidget_calc_intersects_geometry.show()
        # 设置为浮动
        self.dockwidget_calc_intersects_geometry.setFloating(True)

    # 点是否在边界内计算
    def sub_calc_contain_point_in_geometry(self):
        if self.dockwidget_calc_contain_point_in_geometry is None:
            self.dockwidget_calc_contain_point_in_geometry = cyanlove_calc_contain_point_in_geometry()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_calc_contain_point_in_geometry)
            self.dockwidget_calc_contain_point_in_geometry.setGeometry(500, 300, 600, 160)
            # 每次点击时，显示 dockwidget
        self.dockwidget_calc_contain_point_in_geometry.show()
        # 设置为浮动
        self.dockwidget_calc_contain_point_in_geometry.setFloating(True)

    # 相交合并计算
    def sub_calc_dissolve(self):
        if self.dockwidget_calc_dissolve is None:
            self.dockwidget_calc_dissolve = cyanlove_calc_dissolve()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_calc_dissolve)
            self.dockwidget_calc_dissolve.setGeometry(500, 300, 600, 160)
        # 每次点击时，显示 dockwidget
        self.dockwidget_calc_dissolve.show()
        # 设置为浮动
        self.dockwidget_calc_dissolve.setFloating(True)

    def sub_create_index_colname_sqlite(self):
        pass

    @staticmethod
    def sub_analysis2():
        # 这里添加子菜单分析2的逻辑
        msg_box = QtWidgets.QMessageBox()
        msg_box.setWindowTitle("标题")  # 设置窗体标题
        msg_box.setText("功能还没写，后续补充！")  # 设置显示的消息内容
        msg_box.setIcon(QtWidgets.QMessageBox.Information)  # 设置消息框图标
        msg_box.setStandardButtons(QtWidgets.QMessageBox.Ok)  # 添加“确定”按钮
        msg_box.exec_()  # 显示消息框

    def unload(self):
        # 清除菜单和动作
        if self.menu:
            self.iface.mainWindow().menuBar().removeAction(self.menu.menuAction())
            self.menu.deleteLater()  # 释放菜单资源

        # 清除所有动作
        for action in self.actions:
            action.setEnabled(False)  # 禁用动作，以免出现错误
            self.menu.removeAction(action)  # 从菜单中移除动作
        self.actions.clear()  # 清空动作列表
