import csv
import os
import re
from datetime import datetime

import processing
from PyQt5.QtCore import QVariant, QThread
from PyQt5.QtWidgets import QFileDialog
from qgis._core import QgsFeatureRequest, QgsMapLayer, QgsWkbTypes, QgsCoordinateReferenceSystem, \
    QgsCoordinateTransform, QgsProject, QgsGeometry, QgsGeometryCollection, QgsVectorLayer, QgsFields, QgsField, \
    QgsFeature, Qgis, QgsPointXY, QgsUnitTypes, QgsDistanceArea, QgsProcessingContext
from qgis.utils import iface

from qgis.PyQt import QtWidgets, uic, QtCore
from qgis.PyQt.QtCore import pyqtSignal

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'cyanlove_calc_dissolve_base.ui'))


class cyanlove_calc_dissolve(QtWidgets.QDockWidget, FORM_CLASS):
    closingPlugin = pyqtSignal()

    def __init__(self, parent=None):
        super(cyanlove_calc_dissolve, self).__init__(parent)

        self.export_thread = None
        self.iface = iface
        self.canvas = iface.mapCanvas()
        self.setupUi(self)
        self.setFloating(True)
        self.setWindowFlags(QtCore.Qt.Dialog)  # 使对话框始终在前
        # 设置为模态窗口
        # self.setWindowModality(QtCore.Qt.ApplicationModal)  # 或者使用 Qt.WindowModal
        self.btn_calc.clicked.connect(self.calc_dissolve)

    def calc_dissolve(self):
        current_layer = self.mMapLayerComboBox.currentLayer()
        if current_layer is None or current_layer.type() != QgsMapLayer.VectorLayer:
            self.label_show.setText("选择的图层不是矢量图层！")
            return
        geom_type = self.layerselect.wkbType()
        if geom_type not in [QgsWkbTypes.Polygon, QgsWkbTypes.MultiPolygon,
                             QgsWkbTypes.LineString, QgsWkbTypes.MultiLineString]:
            self.label_show.setText("选择的图层不是Polygon图层或者Line图层！")
            return
        # 确保使用现有的进度条
        self.progressBar.setRange(0, 100)  # 设置进度条范围
        self.progressBar.setValue(0)  # 初始化进度条值
        self.progressBar.show()  # 显示进度条 (如果之前是隐藏状态)
        # 开始创建新图层
        timestamp = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
        layname = 'SSJ_CreteDissolve_' + timestamp
        new_layer = QgsProject.instance().mapLayersByName(layname)
        if new_layer:
            for layer in new_layer:
                QgsProject.instance().removeMapLayer(layer.id())
        if geom_type not in [QgsWkbTypes.Polygon, QgsWkbTypes.MultiPolygon]:
            new_layer = QgsVectorLayer("Polygon?crs=EPSG:4326", layname, "memory")
        if geom_type not in [QgsWkbTypes.LineString, QgsWkbTypes.MultiLineString]:
            new_layer = QgsVectorLayer("LineString?crs=EPSG:4326", layname, "memory")

        # 启动导出线程
        self.export_thread = ExportThread(current_layer, new_layer)
        self.export_thread.progress.connect(self.update_progress)  # 连接信号到槽
        self.export_thread.finished.connect(self.export_finished)  # 导出完成后连接到槽
        self.export_thread.start()  # 启动线程

    def update_progress(self, value):
        self.progressBar.setValue(value)  # 更新进度条的值

    def export_finished(self):
        print("导出完成！")
        self.label_show.setText('导出完毕')
        self.progressBar.setValue(100)


class ExportThread(QThread):
    progress = pyqtSignal(int)  # 声明一个信号

    def __init__(self, current_layer, new_layer):
        super().__init__()
        self.current_layer = current_layer
        self.new_layer = new_layer

    def run(self):

        provider = self.new_layer.dataProvider()  # 获取图层的数据提供者
        # 创建处理上下文
        field_names = [f"{field.name()}" for field in self.current_layer.fields()]
        total_features = self.current_layer.featureCount()
        for index, feature in enumerate(self.current_layer.getFeatures()):
            try:
                row = []
                for field_name in field_names:  # 不包括 WKT_BJ
                    value = feature[field_name]  # 获取每个字段的值
                    row.append(f'{value}')  # 在每个值两边添加双引号
                geometry = feature.geometry()
                if geometry:
                    areauint = context.convertAreaMeasurement(context.measureArea(geometry),
                                                              QgsUnitTypes.AreaSquareMeters)
                    row.append(f'{areauint}')  # 添加面积
                # 发送进度更新
                writer.writerow(row)

                progress_value = int((index + 1) / total_features * 100)
                self.progress.emit(progress_value)

            except Exception as e:
                pass


        # 输出结果信息
        print("计算完毕!")
